/*
 * kgraft_patch_bsc1188257
 *
 * Fix for CVE-2021-33909, bsc#1188257
 *
 *  Upstream commit:
 *  None yet.
 *
 *  SLE12-SP3 commit:
 *  77697118294f1846a5d4e72fbf3e9689fd2f86ec
 *
 *  SLE12-SP4, SLE12-SP5, SLE15 and SLE15-SP1 commit:
 *  c848c4293eaf5bd6f72e4515a525a2f721a0fbbb
 *
 *  SLE15-SP2 and -SP3 commit:
 *  eb7ef7694c11d5af5b67113e6d2ec7b70a693f28
 *
 *
 *  Copyright (c) 2021 SUSE
 *  Author: Nicolai Stange <nstange@suse.de>
 *
 *  Based on the original Linux kernel code. Other copyrights apply.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */


/* klp-ccp: from fs/seq_file.c */
#include <linux/fs.h>

/* klp-ccp: from include/linux/fs.h */
#define KGRP_MAX_RW_COUNT (INT_MAX & PAGE_MASK)

/* klp-ccp: from fs/seq_file.c */
#include <linux/export.h>
#include <linux/vmalloc.h>
#include <linux/slab.h>
#include <linux/printk.h>
#include <asm/page.h>

void *kgrp_seq_buf_alloc(unsigned long size)
{
	void *buf;
	gfp_t gfp = GFP_KERNEL;

	/*
	 * Fix CVE-2021-33909
	 *  +3 lines
	 */
	if (unlikely(size > KGRP_MAX_RW_COUNT))
		return NULL;

	/*
	 * For high order allocations, use __GFP_NORETRY to avoid oom-killing -
	 * it's better to fall back to vmalloc() than to kill things.  For small
	 * allocations, just use GFP_KERNEL which will oom kill, thus no need
	 * for vmalloc fallback.
	 */
	if (size > PAGE_SIZE)
		gfp |= __GFP_NORETRY | __GFP_NOWARN;
	buf = kmalloc(size, gfp);
	if (!buf && size > PAGE_SIZE)
		buf = vmalloc(size);
	return buf;
}

#include "kgr_patch_bsc1188257.h"
